import cv2,mediapipe as mp,time
mpdraw=mp.solutions.drawing_utils
mpfacedetction=mp.solutions.face_detection
#,drawSpec=mpdraw.DrawingSpec()
class facedetection():
    def __init__(self,minconfidence=0.5):
        self.faceDetection=mpfacedetction.FaceDetection(minconfidence)
    
    def findfaces(self,img):
        h,w=img.shape[:2]
        imgRGB=cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        results=self.faceDetection.process(imgRGB)
        bboxs=[]
        if results.detections:
            for detection in results.detections:
                pos=detection.location_data.relative_bounding_box
                bbox=int(pos.xmin*w),int(pos.ymin*h),\
                    int(pos.width*w),int(pos.height*h)
                bboxs.append([bbox,detection.score[0]])
        return bboxs



import cv2
from pycv2.img.utils import center_pts
COLORS_OF_WRAPPING = ((0, 0, 255), (240, 0, 159), (255, 0, 0), (255, 255, 0))
def fancydraw(img,bbox,l=30,color=(0,255,0),thickness=10):
    x,y,w,h=bbox
    x1,y1=x+w,y+h
    cv2.line(img,(x,y),(x+l,y),color,thickness)
    cv2.line(img,(x,y),(x,y+l),color,thickness)
    cv2.line(img,(x1,y),(x1-l,y),color,thickness)
    cv2.line(img,(x1,y),(x1,y+l),color,thickness)
    cv2.line(img,(x,y1),(x+l,y1),color,thickness)
    cv2.line(img,(x,y1),(x,y1-l),color,thickness)
    cv2.line(img,(x1,y1),(x1-l,y1),color,thickness)
    cv2.line(img,(x1,y1),(x1,y1-l),color,thickness)
    return img
def drawbox(img,bbox,color=(0,255,0),thickness=3,linetype=1):
    x,y,w,h=bbox
    cv2.rectangle(img,(x,y),((x+w),(y+h)),color,thickness,linetype)
    return img


def draw_box_moving(img,points:list,
        color_line=(255,255,0),thickness=2,radius=2,
        colors:tuple=COLORS_OF_WRAPPING,center_state=False,colorcenter=(0,0,255),radius_center=2):
    img=cv2.cvtColor(img,cv2.COLOR_GRAY2BGR) if len(img.shape)==2 else img
    for i,pt in enumerate(points):
        nexI=(i+1)%len(points)
        cv2.line(img,pt,points[nexI],color_line,thickness)
        
    for i,pt in enumerate(points):
        cv2.circle(img,pt,radius,colors[i%len(colors)],cv2.FILLED)
    if center_state:
        cv2.circle(img,center_pts(points),radius_center,colorcenter,cv2.FILLED)
    return img


import cv2,numpy as np
from pycv2.img.utils import *

def rotate(image,angle):
    center=(int(image.shape[1]/2),int(image.shape[0]/2))
    shape = ( image.shape[1], image.shape[0] ) # cv2.warpAffine expects shape in (length, height)
    matrix = cv2.getRotationMatrix2D( center=center, angle=(angle/np.pi), scale=1 )
    return cv2.warpAffine(image,matrix,shape)
def rotate_all_image(image:np.ndarray, angle:int):
    # grab the dimensions of the image and then determine the
    # center
    (h, w) = image.shape[:2]
    (cX, cY) = (w // 2, h // 2)
    # grab the rotation matrix (applying the negative of the
    # angle to rotate clockwise), then grab the sine and cosine
    # (i.e., the rotation components of the matrix)
    M = cv2.getRotationMatrix2D((cX, cY), -angle, 1.0)
    cos = np.abs(M[0, 0])
    sin = np.abs(M[0, 1])
    # compute the new bounding dimensions of the image
    nW = int((h * sin) + (w * cos))
    nH = int((h * cos) + (w * sin))
    # adjust the rotation matrix to take into account translation
    M[0, 2] += (nW / 2) - cX
    M[1, 2] += (nH / 2) - cY
    # perform the actual rotation and return the image
    return cv2.warpAffine(image, M, (nW, nH))

def rotate_object(pos,cropedimg,src,angle,thresh:None|np.ndarray=None,box:tuple|None=None):
    thresh=thresh if not thresh is None else np.ones(cropedimg.shape[:2],"uint8")*255
    oh,ow=cropedimg.shape[:2]
    img_box=np.zeros((oh,ow),"uint8")
    box=box if not box is None else [0,0,ow,oh]
    cv2.rectangle(img_box,[box[0]-pos[0],box[1]-pos[1]],box[2:],255,cv2.FILLED)

    pos=pos.copy()
    resultimg=src.copy()
    rotated_img=rotate_all_image(cropedimg,angle)
    rotated_mask=rotate_all_image(thresh,angle)
    img_box=rotate_all_image(img_box,angle)
    rotated_mask=cv2.bitwise_and(img_box,rotated_mask)

    h,w=rotated_img.shape[:2]
    #clamping the box
    pos[0]+=(ow-w)//2
    pos[1]+=(oh-h)//2
    _x=0 if pos[0]>0 else abs(pos[0])
    _y=0 if pos[1]>0 else abs(pos[1])
    #to decrease the width and height either
    b=pts_2_xywh(clamp_points(xywh_2_pts([pos[0],pos[1],w,h]),resultimg))
    thresh=rotated_mask.astype(np.bool)[_y:b[3]+_y,_x:b[2]+_x]
    
    resultimg[b[1]:b[1]+b[3],b[0]:b[0]+b[2]][thresh]=rotated_img[_y:b[3]+_y,_x:b[2]+_x][thresh]
    
    return resultimg



from pathlib import PurePath
from pdf2image import convert_from_path
import os,cv2,numpy as np

file_path=os.path.dirname(__file__)
popplerpath=os.path.join(file_path,"\\poppler-0.68.0\\bin")
def savePdf2Img(path,when=0,to=float("inf")):
    images = convert_from_path(path,poppler_path=popplerpath)
    for i,img in enumerate(images):
        if i>=when and i<=to:
            img.save(str(i) +'.jpg', 'JPEG')
def convert_pdf_to_image(path:str | PurePath,first:int,last:int,dpi=200):
    images = []
    images.extend(
                    list(
                        map(
                            lambda image: cv2.cvtColor(
                                np.asarray(image), code=cv2.COLOR_RGB2BGR
                            ),
                            convert_from_path(path, dpi=dpi,poppler_path=popplerpath,first_page=first,last_page=last),
                        )
                    )
                )
    return images




import cv2,mediapipe as mp,math

mppose=mp.solutions.pose
mpDraw=mp.solutions.drawing_utils
class poseDetctor():
    def __init__(self,mode=False,upperbody=False,smoothlandmarks=False,min_detect_confid=0.5,min_track_confid=0.5,drawSpec=mpDraw.DrawingSpec()):
        self.upper=upperbody
        self.drawSpec=drawSpec
        self.pose=mppose.Pose(mode,upperbody,smoothlandmarks,min_detect_confid,min_track_confid)
    def findPose(self,img,draw=True):
        imgRGB=cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        results=self.pose.process(imgRGB)
        if draw and results.pose_landmarks:
            if self.upper:
                mpDraw.draw_landmarks(img,results.pose_landmarks,mppose.UPPER_BODY_POSE_CONNECTIONS,self.drawSpec,self.drawSpec)
            else:
                mpDraw.draw_landmarks(img,results.pose_landmarks,mppose.POSE_CONNECTIONS,self.drawSpec,self.drawSpec)
            
        return img
    def findpositions(self,img,draw=True,color=(255,0,0),radius=5):
        imgRGB=cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        results=self.pose.process(imgRGB)
        lmlist=[]
        if  results.pose_landmarks:
            for lm in results.pose_landmarks.landmark:
                h,w,c=img.shape
                cx,cy=int(lm.x*w),int(lm.y*h)
                lmlist.append([cx,cy])
                if draw:
                    cv2.circle(img,(cx,cy),radius,color,cv2.FILLED)
        return lmlist
    def get_results(self,img):
        return self.pose.process(cv2.cvtColor(img,cv2.COLOR_BGR2RGB))

    def findAngel(self,img,p1,p2,p3,draw=True,color=(255,0,0),radius=5,lcolor=(255,255,255),thickness=1,drawtext=True,fontscale=5,font=cv2.FONT_HERSHEY_SIMPLEX,tcolor=(0,255,0),textthickness=1):
        lmlist=self.findpositions(img,False)
        x1,y1=lmlist[p1];x2,y2=lmlist[p2];x3,y3=lmlist[p3]
        angel=math.degrees(math.atan2(y3-y2,x3-x2)-
                math.atan2(y1-y2,x1-x2))
        if angel>180:angel=360-angel
        elif angel<0:angel+=360
        if draw:
            cv2.line(img,lmlist[p2],lmlist[p1],lcolor,thickness)
            cv2.line(img,lmlist[p2],lmlist[p3],lcolor,thickness)
            for center in [lmlist[p1],lmlist[p2],lmlist[p3]]:
               cv2.circle(img,center,int(radius/2),color,cv2.FILLED)
               cv2.circle(img,center,int(radius),color)
            if drawtext:
                cv2.putText(img,str(int(angel)),(x2-10,y2),font,fontscale,tcolor,textthickness)
        return angel
    
    def findLenght(self,img,p1,p2,draw=True,color=(255,0,0),radius=5,lcolor=(255,255,2555),thickness=3):
        lmlist=self.findpositions(img,False)
        x1,y1=lmlist[p1][0],lmlist[p1][1]
        x2,y2=lmlist[p2][0],lmlist[p2][1]
        cx,cy=(x1+x2)//2,(y1+y2)//2
        lenght=math.hypot(x2-x1,y2-y1)  
        if draw:
            cv2.line(img,(x1,y1),(x2,y2),lcolor,thickness)   
            cv2.circle(img,(x1,y1),radius,color,thickness,cv2.FILLED)
            cv2.circle(img,(x2,y2),radius,color,thickness,cv2.FILLED)
            cv2.circle(img,(cx,cy),radius,color,thickness,cv2.FILLED)
        return lenght
        

    



import mediapipe as mp
import cv2
import math
from google.protobuf.json_format import MessageToDict
mpDraw = mp.solutions.drawing_utils
mp_hand = mp.solutions.hands
HAND_LANDMARKS_LENGTH = 21
HAND_CONNECTIONS = mp_hand.HAND_CONNECTIONS
LEFT_HAND = "left"
RIGHT_HAND = "right"
tipIds = [4, 8, 12, 16, 20]


class Hand_detector(mp_hand.Hands):
    def __init__(self, mode=False, maxhands=2, detectioncon=0.5, trackcon=0.5, drawSpec=mpDraw.DrawingSpec()):
        super().__init__(mode, maxhands, detectioncon, trackcon)
        self.drawSpec = drawSpec
        self.results = None

    def find_hands(self, img):
        img.flags.writeable = False
        imgRgb = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        self.result = self.process(imgRgb)
        img.flags.writeable = True
        return self.result

    def draw_land_marks(self, img, results=None):
        if results is None:
            results = self.result
        if results.multi_hand_landmarks:
            for handLms in results.multi_hand_landmarks:
                mpDraw.draw_landmarks(
                    img, handLms, HAND_CONNECTIONS, mpDraw.DrawingSpec(), mpDraw.DrawingSpec())
        return img

    def hands_type(self, results=None):
        if results is None:
            results = self.result
        hand_type = {}

        def convert(hand):
            return MessageToDict(hand)["classification"][0]
        if results.multi_handedness:
            hand_classification = list(map(convert, results.multi_handedness))
            for num, hand in enumerate(hand_classification):
                label = hand["label"]
                if label not in hand_type:
                    hand_type[label] = num
                else:
                    other_hand = hand_classification[hand_type[label]]["score"]
                    if other_hand > hand["score"]:
                        hand_type[label] = num
        return hand_type

    def findpositions(self, img, handnum=0, draw=True):
        lmlist = []
        if self.result.multi_hand_landmarks:
            handLms = self.result.multi_hand_landmarks[handnum]
            for lm in handLms.landmark:
                h, w, c = img.shape
                cx, cy = int(lm.x*w), int(lm.y*h)
                lmlist.append([cx, cy])
                if draw:
                    cv2.circle(img, (cx, cy), 25, (255, 0, 255), cv2.FILLED)
        return lmlist

    def findlength(self, img, p1, p2, draw=True, color=(255, 0, 0), radius=5, lcolor=(255, 255, 2555), thickness=3):
        lmlist = self.findpositions(img, False)
        x1, y1 = lmlist[p1][0], lmlist[p1][1]
        x2, y2 = lmlist[p2][0], lmlist[p2][1]
        cx, cy = (x1+x2)//2, (y1+y2)//2
        length = math.hypot(x2-x1, y2-y1)
        if draw:
            cv2.line(img, (x1, y1), (x2, y2), lcolor, thickness)
            cv2.circle(img, (x1, y1), radius, color, thickness, cv2.FILLED)
            cv2.circle(img, (x2, y2), radius, color, thickness, cv2.FILLED)
            cv2.circle(img, (cx, cy), radius, color, thickness, cv2.FILLED)
        return length

    def findAngel(self, img, p1, p2, p3, draw=True, color=(255, 0, 0), radius=5, lcolor=(255, 255, 255), thickness=1, drawtext=True, fontscale=5, font=cv2.FONT_HERSHEY_SIMPLEX, tcolor=(0, 255, 0), textthickness=1):
        lmlist = self.findpositions(img, False)
        x1, y1 = lmlist[p1]
        x2, y2 = lmlist[p2]
        x3, y3 = lmlist[p3]
        angel = math.degrees(math.atan2(y3-y2, x3-x2) -
                             math.atan2(y1-y2, x1-x2))
        if angel > 180:
            angel = 360-angel
        elif angel < 0:
            angel += 360
        if draw:
            cv2.line(img, lmlist[p2], lmlist[p1], lcolor, thickness)
            cv2.line(img, lmlist[p2], lmlist[p3], lcolor, thickness)
            for center in [lmlist[p1], lmlist[p2], lmlist[p3]]:
                cv2.circle(img, center, int(radius/2), color, cv2.FILLED)
                cv2.circle(img, center, int(radius), color)
            if drawtext:
                cv2.putText(img, str(int(angel)), (x2-10, y2),
                            font, fontscale, tcolor, textthickness)
        return angel



import cv2,numpy as np
from imutils import contours
import imutils
from pycv2.img.utils import *
from pycv2.img.roate_img import *
from pycv2.img.conv_pdf import *


COLORS = ((0, 0, 255), (240, 0, 159), (255, 0, 0), (255, 255, 0))
def get_contours(image:np.ndarray):
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    gray = cv2.GaussianBlur(gray, (7, 7), 0)
    edged = cv2.Canny(gray, 50, 100)
    edged = cv2.dilate(edged, None, iterations=1)
    edged = cv2.erode(edged, None, iterations=1)
    cnts = cv2.findContours(edged, cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)
    cnts = imutils.grab_contours(cnts)
    (cnts, _) = contours.sort_contours(cnts)
    return cnts 

def get_contours_boxs(image:np.ndarray,cnts:tuple,minarea=1000,draw=False):
        c=None;i=None;bigarea=0
        for (i_small, c_small) in enumerate(cnts):
            if cv2.contourArea(c_small) > minarea and bigarea<cv2.contourArea(c_small):
                bigarea=cv2.contourArea(c_small)
                c=c_small;i=i=i_small
                
        box = cv2.minAreaRect(c)
        box = cv2.cv.BoxPoints(box) if imutils.is_cv2() else cv2.boxPoints(box)
        box = np.array(box, dtype="int")
        rect = order_points(box)
        if draw:
            cv2.drawContours(image, [box], -1, (0, 255, 0), 2)
            for ((x, y), color) in zip(rect, COLORS):
                cv2.circle(image, (int(x), int(y)), 5, color, -1)

            cv2.putText(image, "Object #{}".format(i + 1),
                (int(rect[0][0] - 15), int(rect[0][1] - 15)),
                cv2.FONT_HERSHEY_SIMPLEX, 0.55, (255, 255, 255), 2)
        return image,box
def get_all_contours_boxs(image,cnts,minarea: int=1000,draw=False):
        boxs=[]
        for (i, c) in enumerate(cnts):
            if cv2.contourArea(c) < minarea:
                continue
            box = cv2.minAreaRect(c)
            box = cv2.cv.BoxPoints(box) if imutils.is_cv2() else cv2.boxPoints(box)
            box = np.array(box, dtype="int")
            rect = order_points(box)
            boxs.append(box)
            if draw:
                cv2.drawContours(image, [box], -1, (0, 255, 0), 2)
                for ((x, y), color) in zip(rect, COLORS):
                    cv2.circle(image, (int(x), int(y)), 5, color, -1)

                cv2.putText(image, "Object #{}".format(i + 1),
                    (int(rect[0][0] - 15), int(rect[0][1] - 15)),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.55, (255, 255, 255), 2)
        return image,boxs
def transparentOverlay(src:np.ndarray, overlay:np.ndarray, pos=(0, 0), scale=1):
    overlay = cv2.resize(overlay, (0, 0), fx=scale, fy=scale)
    h, w, _ = overlay.shape  # Size of foreground
    rows, cols, _ = src.shape  # Size of background Image
    y, x = pos[0], pos[1]  # Position of foreground/overlay image
    # loop over all pixels and apply the blending equation
    for i in range(h):
        for j in range(w):
            if x + i >= rows or y + j >= cols:
                continue
            alpha = float(overlay[i][j][3] / 255.0)  # read the alpha channel
            src[x + i][y + j] = alpha * overlay[i][j][:3] + (1 - alpha) * src[x + i][y + j]
    return src

def cluster_img(img:np.ndarray,k:int):
    criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 10, 1.0)
    flags = cv2.KMEANS_RANDOM_CENTERS
    Z = img.reshape((-1,3))
    # convert to np.float32
    Z = np.float32(Z)
    _,label,center = cv2.kmeans(Z,k,None,criteria,10,flags)
    center = np.uint8(center)
    res = center[label.flatten()]
    return res.reshape((img.shape))




import cv2
def draw_text(img,text:str,org, fontFace, fontScale,color, thickness):
    x,y=org
    for line in text.splitlines():
        cv2.putText(img,line,(x,y),fontFace, fontScale,color, thickness)
        (_,h),base_line=cv2.getTextSize(line,fontFace, fontScale, thickness)
        y+=base_line+h
def text_size(text:str,fontFace, fontScale, thickness):
    current_width=0
    currentheight=0
    for line in text.splitlines():
        (w,h),base_line=cv2.getTextSize(line,fontFace=fontFace,fontScale=fontScale,thickness=thickness)
        current_width=w if w>current_width else current_width
        currentheight+=(h+base_line)
    return current_width,currentheight
def text2lines(text:str,width,
    fontFace=cv2.FONT_HERSHEY_SIMPLEX,fontScale=1,thickness=1,
    size_charcter=(None,None),
    break_=False,):
        new_text=""
        for line in text.splitlines():
            total_width=0
            if break_:
                for letter in line:
                    width_charcter=size_charcter[0] if size_charcter!=(None,None) else cv2.getTextSize(letter,fontFace=fontFace,fontScale=fontScale,thickness=thickness)[0][0]
                    if total_width+width_charcter>width:
                        total_width=width_charcter
                        new_text+="\n"
                    else:
                        total_width+=width_charcter
                    new_text+=letter
                new_text+="\n"
            else:
                space_width=size_charcter[0] if size_charcter!=(None,None) else cv2.getTextSize(" ",fontFace=fontFace,fontScale=fontScale,thickness=thickness)[0][0]
                for word in line.split(" "):

                    width_charcter=size_charcter[0] if size_charcter!=(None,None) else cv2.getTextSize(word,fontFace=fontFace,fontScale=fontScale,thickness=thickness)[0][0]

                    if total_width+width_charcter>width:
                        total_width=width_charcter
                        new_text+="\n"
                    else:
                        total_width+=width_charcter+space_width
                    #space for next word
                    new_text+=word+" "
            new_text+="\n"
        return new_text



import cv2,mediapipe as mp

#cap=cv2.VideoCapture("http://192.168.0.104:8080/video")
class Face_landmarks():
    mpdraw=mp.solutions.drawing_utils
    mpfacemesh=mp.solutions.face_mesh
    def __init__(self,mode=False,max_num_faces=1,detect_confid=0.5,track_confid=0.5,drawSpec=mpdraw.DrawingSpec()):
        self.drawSpec=drawSpec
        self.facemesh=Face_landmarks.mpfacemesh.FaceMesh(mode,max_num_faces,detect_confid,track_confid)
    def findface(self,img,draw=True):
        imgRGB=cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        results=self.facemesh.process(imgRGB)
        if draw and results.multi_face_landmarks:
            for facelms in results.multi_face_landmarks:
                Face_landmarks.mpdraw.draw_landmarks(img,facelms,Face_landmarks.mpfacemesh.FACE_CONNECTIONS,
                self.drawSpec,self.drawSpec)
        return img
    def findpositions(self,img,draw=True,color=(255,0,0),radius=5):
        h,w=img.shape[:2]
        imgRGB=cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        results=self.facemesh.process(imgRGB)
        lmlist=[]
        if results.multi_face_landmarks:
            for facelms in results.multi_face_landmarks:
                for id,lm in enumerate(facelms.landmark):
                    cx,cy=(int(lm.x*w)),(int(lm.y*h))
                    lmlist.append([id,cx,cy])
                    if draw:
                        cv2.circle(img,(cx,cy),radius,color,cv2.FILLED)
        return lmlist




import cv2
from pycv2.img.drawing import fancydraw

class objectdetctor():
    def __init__(self,img,bbox):
        self.tracker=cv2.TrackerCSRT_create()
        self.tracker.init(img, bbox)
    def update(self,img):
        success,bbox=self.tracker.update(img)
        if success:
            bbox=int(bbox[0]),int(bbox[1]),int(bbox[2]),int(bbox[3])
        return bbox
    def fancydraw(self,img,bbox,l=30,color=(0,255,0),thickness=10):
        return fancydraw(img,bbox,l,color,thickness)




import cv2
import time
from pycv2.tools.utils import *

def get_cam_properties(cap):
    """
    return dictionary of fps and frame count
    """
    fps = cap.get(cv2.CAP_PROP_FPS)      # OpenCV2 version 2 used "CV_CAP_PROP_FPS"
    frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
    return {"fps":fps,"frame_count":frame_count}
def show_cam_prop(cap:cv2.VideoCapture):
    fps,frame_count=list(get_cam_properties(cap).values())
    duration = frame_count/fps
    print('fps = ' + str(fps))
    print('number of frames = ' + str(frame_count))
    print('duration (S) = ' + str(duration))
    minutes = int(duration/60)
    seconds = duration%60
    print('duration (M:S) = ' + str(minutes) + ':' + str(seconds))
def video_saver(file_name,video_capture,fourcc=cv2.VideoWriter_fourcc("m","p","4","v")
    ,start_time=0,end_time=float("inf"),fps_v=None,funct=None):
    h,w=video_capture.read()[1].shape[:2]
    
    fps,frame_count=list(get_cam_properties(video_capture).values())
    duration = frame_count/fps
    end_time=min(end_time,duration)
    start_time=min(start_time,end_time)
    start_frame=int(fps*start_time)
    end_frame=int(fps*end_time)
    video_capture.set(cv2.CAP_PROP_POS_FRAMES, start_frame)
    if fps_v is None:
        fps_v=fps
    out=cv2.VideoWriter(file_name,fourcc,fps_v,(w,h))
    for _ in progressBar(range(start_frame,end_frame), prefix = 'Progress:', suffix = 'Complete', length = 50):
        success,frame=video_capture.read()
        if not success:break
        if not funct is None:
            frameNumber=video_capture.get(cv2.CAP_PROP_POS_FRAMES)
            frame=funct(frame,frameNumber,fps)
        out.write(frame)
    out.release()
def conver_frames2video(file_name,
        frames,fourcc=cv2.VideoWriter_fourcc("m","p","4","v"),
    fps_v=20,funct=None):
    h,w=frames[0].shape[:2]
    out=cv2.VideoWriter(file_name,fourcc,fps_v,(w,h))
    for id,frame in enumerate(progressBar(frames, prefix = 'Progress:', suffix = 'Complete', length = 50)):
        if not funct is None:
            frame=funct(frame,frameNumber=id,fps=fps_v)
        out.write(frame)
    out.release()
#fps the number of frames per seconds
class FPS():
    def __init__(self):
        self.start_t=0
    @staticmethod
    def calculate_fps(time_passed,frames):
        return frames/time_passed
    def start(self):
        self.start_t=time.time()
    def end(self,frame=None):
        starttime=self.start_t
        self.start_t=time.time()
        fps=int(1/(time.time()-starttime))
        if not frame is None:
            cv2.putText(frame,str(fps),(20,50),cv2.FONT_HERSHEY_COMPLEX,2,(0,255,0))
        return fps





def createProgressBar(prefix = '', suffix = '', decimals = 1, length = 70, fill = '█', printEnd = "\r",total=100):
    def printProgressBar (iteration):
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        filledLength = int(length * iteration // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    return printProgressBar
    
def progressBar(iterable, prefix = '', suffix = '', decimals = 1, length = 70, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iterable    - Required  : iterable object (Iterable)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    total = len(iterable)
    printProgressBar=createProgressBar(prefix, suffix, decimals, length , fill, printEnd,total )
    printProgressBar(0)
    # Update Progress Bar
    for i, item in enumerate(iterable):
        yield item
        printProgressBar(i + 1)
    # Print New Line on Complete
    print()
    


import cv2 ,numpy as np
from PIL import Image
def pil2cv(src):
    img=np.array(src)
    return cv2.cvtColor(img,cv2.COLOR_RGBA2BGR)
def cv2pil(src):
    img=cv2.cvtColor(src,cv2.COLOR_BGR2RGB)
    return Image.fromarray(img)



from collections.abc import Iterable
import cv2,numpy as np
from scipy.spatial import distance as dist
import math
import re

        
def remove_back_ground(mask,img):
    threshimg= cv2.threshold(mask, 0, 255, 0)[1]
    return cv2.bitwise_and(img,img,mask=255-threshimg)

def duplicate_image(img,shape):
    "shape is height and width"
    h,w=shape
    newimage=np.zeros((h,w,3),"uint8")
    ih,iw=img.shape[:2]
    start=[0,0]
    while start!=[h,w]:
        start[1]=0
        y=min(h,start[0]+ih)
        cy=y-start[0]
        while start[1]!=w:
            x=min(w,start[1]+iw)
            cx=x-start[1]
            newimage[start[0]:y,start[1]:x]=img[0:cy,0:cx]
            start[1]=x
        start[0]=y
        
    return newimage

def clamp_box(box,imgcv):
    box=list(box)
    h,w=imgcv.shape[:2]
    box[0]=min(w,max(0,box[0]))
    box[1]=min(h,max(0,box[1]))
    box[2]=max(0,min(box[0]+box[2],w)-box[0])
    box[3]=max(0,min(box[1]+box[3],h)-box[1])
    return box
def clamp_points(points: list,imgcv):
    points=list(points.copy())
    
    for id,pt in enumerate(points):
        points[id]=clamp_point(pt,imgcv)
    return points
    
def clamp_point(pt,imgcv):
    h,w=imgcv.shape[:2]
    pt=list(pt)
    pt[0]=max(0,min(int(pt[0]),w-1))
    pt[1]=max(0,min(int(pt[1]),h-1))
    return pt
def centerbox(box):
    return box[0]+int(box[2]/2),box[1]+int(box[3]/2)
def center_pts(pts):
    return centerbox(pts_2_xywh(pts))
def compare_maskes(mask1,mask2):
    if mask1.shape==mask2.shape:
        newmask=cv2.bitwise_xor(mask1,mask2)
        for color in cv2.split(newmask):
            return (cv2.countNonZero(color)/(mask1.shape[1]*mask1.shape[0]))*100        
    else:
        raise "the makses are not the same shape "
def ARE_EQUALE(img1,img2):
    if img1 is None or img2 is None:return False
    if img1.shape==img2.shape:
        return not np.any(cv2.absdiff(img1,img2))
    else:
        return False


def convert_tuple_to_list(object):
    if isinstance(object,Iterable):
        object=list(object)
        for id,x in enumerate(object):
            object[id]=convert_tuple_to_list(x)
    return object
def conv_pts_to_np(pts): 
    return np.array(pts)
def conv_np_to_pts(circles):
    if type(circles)==np.ndarray:
        pts=[0,0,0,0]
        for n in range(4):
            pts[n]=[int(circles[n][0]),int(circles[n][1])]
        return pts
    else:raise "the is not numpy array "
def order_points(pts):
	xSorted = pts[np.argsort(pts[:, 0]), :]
	leftMost = xSorted[:2, :]
	rightMost = xSorted[2:, :]
	leftMost = leftMost[np.argsort(leftMost[:, 1]), :]
	(tl, bl) = leftMost
	D = dist.cdist(tl[np.newaxis], rightMost, "euclidean")[0]
	(br, tr) = rightMost[np.argsort(D)[::-1], :]
	return np.array([tl, tr, br, bl], dtype="float32")
def four_point_transform(src, pts):
    rect = pts
    (tl, tr, br, bl) = rect
    widthA = np.sqrt(((br[0] - bl[0]) ** 2) + ((br[1] - bl[1]) ** 2))
    widthB = np.sqrt(((tr[0] - tl[0]) ** 2) + ((tr[1] - tl[1]) ** 2))
    maxWidth = max(int(widthA), int(widthB))
    heightA = np.sqrt(((tr[0] - br[0]) ** 2) + ((tr[1] - br[1]) ** 2))
    heightB = np.sqrt(((tl[0] - bl[0]) ** 2) + ((tl[1] - bl[1]) ** 2))
    maxHeight = max(int(heightA), int(heightB))
    dst = np.array([
        [0, 0],
        [maxWidth - 1, 0],
        [maxWidth - 1, maxHeight - 1],
        [0, maxHeight - 1]], dtype = "float32")
    M = cv2.getPerspectiveTransform(rect, dst)
    warped = cv2.warpPerspective(src, M, (maxWidth, maxHeight))
    return warped
def padimage(img,pad):
    return img[pad:img.shape[0]-pad,pad:img.shape[1]-pad]

def rectContains(rect,pt):
    logic = rect[0] < pt[0] < rect[0]+rect[2] and rect[1] < pt[1] < rect[1]+rect[3]
    return logic
def resize_img(src, width = None, height = None,precent=None, inter = cv2.INTER_AREA):
    if precent!=None:
        return cv2.resize(src,(0,0),None,precent,precent,inter=inter)
    dim = None
    (h, w) = src.shape[:2]
    if width is None and height is None:
        return src
    if width is None:
        r = height / float(h)
        dim = (int(w * r), height)
    else:
        r = width / float(w)
        dim = (width, int(h * r))
    resized = cv2.resize(src, dim, interpolation = inter)
    return resized
def closest_node_index(node, nodes,maxdistance=float("inf")):
    points,pos=np.array(nodes),np.array(node)
    dist=np.sqrt(np.sum((points-pos)**2,axis=1))
    pt=np.argmin(dist)
    if dist[pt]>maxdistance:
        return None
    else:
        return pt,dist[pt]

def closest_node(node, nodes,maxdistance=float("inf")):
    close=closest_node_index(node, nodes,maxdistance)
    if close!=None:
        return close[1]
def all_closetest_nodes(node, nodes,maxdistance):
    points,pos=np.array(nodes),np.array(node)
    dist=np.sqrt(np.sum((points-pos)**2,axis=1))
    return list(points[np.where(dist<maxdistance)])

def distance(pt_1, pt_2):
    pt_1 = np.array((pt_1[0], pt_1[1]))
    pt_2 = np.array((pt_2[0], pt_2[1]))
    return np.linalg.norm(pt_1-pt_2)
def xywh_2_pts(box):
    return [
    [box[0],box[1]],
    [box[0]+box[2],box[1]],
    [box[0]+box[2],box[1]+box[3]],
    [box[0],box[1]+box[3]],
    ]
def pts_2_xywh(pts):
    if len(pts)<=3:
        raise "the points are not four points"
    x,y=float("inf"),float("inf")
    mx,my=0,0
    for pos in pts:
        if pos[0]<x:
            x=int(pos[0])
        if pos[0]>mx:
            mx=int(pos[0])
        if pos[1]<y:
            y=int(pos[1])
        if pos[1]>my:
            my=int(pos[1])
    w,h=(mx-x),(my-y)
    return [x,y,w,h]    
    
def organize_pts(pts):
    if len(pts)==4:
        circles=np.array(pts,np.float)
        circles=order_points(circles)
        circles=circles.astype(int)
        circles=list(circles)
        return circles
    else:raise "This must be only 4 points"
def rgb_to_hex(r, g, b):
    def clamp(x): 
        return max(0, min(x, 255))
    return "#{0:02x}{1:02x}{2:02x}".format(clamp(r), clamp(g), clamp(b))
def hex_to_rgb(hexcolor:str):
    match=re.search(r'^#(?:[0-9a-fA-F]{3}){1,2}$', str(hexcolor))
    if match:
        h = hexcolor.lstrip('#')
        return tuple(int(h[i:i+2], 16) for i in (0, 2, 4))
    else:raise "the hex color is not correct"
def get_transparency_image(img,thresh):
    b, g, r = cv2.split(img)
    rgba = [b,g,r, thresh]
    dst = cv2.merge(rgba,4)
    return dst
def get_containing_boxes(all_points):
    positions=[[],[]]
    for points in all_points:
        for pt in points:
            positions[0].append(pt[0])
            positions[1].append(pt[1])
    min_x,min_y=sorted(positions[0])[0],sorted(positions[1])[0]
    max_x=sorted(positions[0],reverse=True)[0]
    max_y=sorted(positions[1],reverse=True)[0]
    return min_x,min_y,(max_x-min_x),(max_y-min_y)

def newbox(threshimg,box=None):
    box=box if not box is None else [0,0,threshimg.shape[1],threshimg.shape[0]] 
    if cv2.countNonZero(threshimg)==0:return box
    contours = cv2.findContours(threshimg,cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_SIMPLE)[0]
    if len(contours)!=0:
        all_points=[]
        for cnt in contours: 
            all_points.append(xywh_2_pts(cv2.boundingRect(cnt)))
        the_new_box=list(get_containing_boxes(all_points))
        the_new_box[0]+=box[0];the_new_box[1]+=box[1]
        return the_new_box
    return box

def get_rotated_box(box,theta,center=None):
    c=center if not center is None else centerbox(box)
    new_pts=[]
    for pt in xywh_2_pts(box[:4]):
        new_pts.append(get_rotated_point(c,pt,theta))
    return pts_2_xywh(new_pts)
def get_rotated_pts(pts,theta,center=None):
    c=center if not center is None else center_pts(pts)
    new_pts=[]
    for pt in pts:
        new_pts.append(get_rotated_point(c,pt,theta))
    return new_pts
def get_rotated_point(center,pt,theta):
    rotated_x = math.cos(theta) * (pt[0] - center[0]) - math.sin(theta) * (pt[1] - center[1]) + center[0]
    rotated_y = math.sin(theta) * (pt[0] - center[0]) + math.cos(theta) * (pt[1]- center[1]) + center[1]
    return [int(rotated_x),int(rotated_y)]
def pyramids(img,num:int):
    gussing_pyramid=img.copy()
    if num>0:
        for x in range(0,num):
            gussing_pyramid=cv2.pyrUp(gussing_pyramid)
    elif num<0:
        for x in range(abs(num),0,-1):
            gussing_pyramid=cv2.pyrDown(gussing_pyramid)
    return gussing_pyramid

def color_back_ground(src,mask,color:tuple):
    coloredbackground=src.copy()
    coloredbackground[mask.astype(np.bool)]=color
    return coloredbackground
def add_back_ground(src,mask:np.ndarray,another_img:np.ndarray):
    src=src.copy()
    mask=mask.astype(np.bool)
    src[mask]=another_img[mask]
    return src
def bluring(src,mask,radius=7):
    src=src.copy()
    bluredimg=cv2.blur(src,(radius,radius))
    return add_back_ground(src,mask,bluredimg)
def isgray(img):
    if len(img.shape) < 3: return True
    if img.shape[2]  == 1: return True
    b,g,r = img[:,:,0], img[:,:,1], img[:,:,2]
    if (b==g).all() and (b==r).all(): return True
    return False



from PIL.ImageDraw import ImageDraw


class TextDrawer(ImageDraw):
    def text2lines(self,
        text:str,
        width=9999999,
        font=None,
        spacing=4,
        direction=None,
        features=None,
        language=None,
        stroke_width=0,break_=False):
        new_text=""
        lines=text.splitlines()
        for i,line in enumerate(lines):
            total_width=0
            if break_:
                for letter in line:
                    width_charcter=self.textsize(letter,font, spacing, direction, features, language, stroke_width)[0]
                    if total_width+width_charcter>width:
                        total_width=width_charcter
                        new_text+="\n"
                    else:
                        total_width+=width_charcter
                    new_text+=letter
            else:
                space_width=self.textsize(" ",font, spacing, direction, features, language, stroke_width)[0]
                words=line.split(" ")
                
                for iw,word in enumerate(words):
                    width_charcter=self.textsize(word,font, spacing, direction, features, language, stroke_width)[0]
                    if total_width+width_charcter>width:
                        total_width=width_charcter
                        new_text+="\n"
                    else:
                        total_width+=width_charcter+space_width
                    new_text+=word
                    #space for next word
                    if iw!=len(words)-1:
                        new_text+=" "      
            #to ensure that he will not add a space at the end of the text
            if i!=len(lines)-1:
                new_text+="\n"
        return new_text

def Draw(im, mode=None):
    try:
        return im.getdraw(mode)
    except AttributeError:
        return TextDrawer(im, mode)


import pytesseract,cv2,numpy as np
from pytesseract import Output 
Tesseract_file=None
def init_tesseract_path(filePath):
    global Tesseract_file
    Tesseract_file=filePath    
def read_text(mask):
    d=get_img_infromations(mask)
    dic=[]
    for i in range(len(d['text'])):
        small_dic={}
        for x in d:
            small_dic[x]=d[x][i]
        dic.append(small_dic)
    text=""
    lasline=1
    for val in dic:
        if val["conf"]>60:
            if int(val["lin_num"])>lasline:
                lasline=int(val["lin_num"])
                text+="\n"
            text+=val["text"]+" "          
    return text
def get_img_infromations(img):
    if not Tesseract_file:
        return print("YOU MUST INITIALIZE TESSERACT PATH FRIST")
    pytesseract.pytesseract.tesseract_cmd=Tesseract_file
    return  pytesseract.image_to_data(img,output_type=Output.DICT)
def get_img_boxes(img):
    if not Tesseract_file:
        return print("YOU MUST INITIALIZE TESSERACT PATH FRIST")
    pytesseract.pytesseract.tesseract_cmd=Tesseract_file
    h_img=img.shape[0]
    d=pytesseract.image_to_boxes(img,output_type=Output.DICT)
    boxes=[]
    n_boxes = len(d['char'])
    for i in range(n_boxes):
        (text,x1,y2,x2,y1) = (d['char'][i],d['left'][i],d['top'][i],d['right'][i],d['bottom'][i])
        text=text
        x,y=int(x1),int(h_img-int(y2))
        w,h=int(int(x2)-x),int(h_img-int(y1)-y)
        boxes.append((text,(x,y,w,h)))
    return boxes
def get_img_string(img):
    if not Tesseract_file:
        return print("YOU MUST INITIALIZE TESSERACT PATH FRIST")
    pytesseract.pytesseract.tesseract_cmd=Tesseract_file
    return pytesseract.image_to_string(img,lang="auto")
def get_text_mask(img):
        mask=np.zeros(img.shape[:2], dtype=np.uint8)
        dic=[]
        d=get_img_boxes(img)
        for _,box in d:
            small_dic={"conf":100}
            small_dic["box"]=box
            dic.append(small_dic)
        wide=3
        for val in dic:
            if int(val["conf"])>60:
                x,y,w,h=val["box"]
                x,y,w,h=max(0,x-wide),max(0,y-wide),min(mask.shape[1],w+wide),min(mask.shape[0],h+wide)
                gray = cv2.cvtColor(img[y:y+h,x:x+w],cv2.COLOR_BGR2GRAY)
                edges = cv2.Canny(gray, 0, 255)
                mask[y:y+h,x:x+w]=edges
        mask = cv2.dilate(mask, None,iterations=2)
        contours = cv2.findContours(mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE)[0]
        for contour in contours:
            cv2.fillConvexPoly(mask, contour, (255))
        mask=cv2.erode(mask, None,iterations=2)
        return mask












from setuptools import setup, find_packages
VERSION = '0.0.1.1'
DESCRIPTION = 'Lightweight utility package for common computer vision tasks.'
LONG_DESCRIPTION = 'A series of convenience functions to make basic image processing functions such as translation, rotation, resizing, skeletonization, displaying Matplotlib images, sorting contours, detecting edges, and much more easier with OpenCV and both Python 2.7 and Python 3.'
import os

def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    print(paths)
    return paths
# Setting up
setup(
    name="cv-imutils",
    version=VERSION,
    author="Emam_ahsour",
    author_email="emam54637@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=["opencv-python", "numpy", "mediapipe",
                      "Pillow", "scipy", "imutils", "pdf2image", "pytesseract"],
    keywords=['python', 'cv2-utils', 'tools', 'pycv2',
              'cv2-functions', "cv2-most-used functions"],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    package_data={'': package_files("./pycv2/img/poppler")},
    include_package_data=True,
)
